import fsp from 'node:fs/promises';
import Ajv from 'ajv';
const ajv = new Ajv();

const config_validate = ajv.compile({
    type: 'object',
    properties: {
        clients: {
            type: 'array',
            items: {
                type: 'object',
                properties: {
                    name: { type: 'string' },
                    keySha256Sum: { type: 'string' },
                    quota: { type: 'number', nullable: true },
                },
                required: ['name', 'keySha256Sum'],
            },
        },
        storePath: { type: 'string' },
    },
    required: ['clients', 'storePath'],
});

export class Config {
    clients: {
        name: string;
        keySha256Sum: string;
        /**
         * In megabytes
         */
        quota?: number;
    }[];
    storePath: string;

    static async load(): Promise<Config> {
        const path = process.env.CONFIG_PATH ?? 'config.json';
        const raw = JSON.parse(
            await fsp.readFile(path, { encoding: 'utf8' }),
        ) as Config;
        if (!config_validate(raw)) {
            throw (
                'Config invalid: ' +
                (config_validate.errors
                    ?.map((x) => x.keyword + ': ' + x.message)
                    ?.join('; ') ?? 'No validation errors provided')
            );
        }

        try {
            await fsp.mkdir(raw.storePath, { recursive: true });
        } catch (err) {
            console.error('Could not create storePath directory:');
            throw err;
        }

        return raw;
    }
}

export const config = await Config.load();
